using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Text.RegularExpressions;

namespace SpringClient
{
	/// <summary>
	/// Summary description for BattleLoginPrompt.
	/// </summary>
	public class BattleLoginPrompt : System.Windows.Forms.Form
	{
		private System.Windows.Forms.TextBox textBoxPassword;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.CheckBox checkBox1;
		private System.Windows.Forms.Button buttonOK;
		private System.Windows.Forms.Button buttonCancel;
		private System.Windows.Forms.ErrorProvider errorProviderPassword;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public string Password
		{
			get
			{
				return textBoxPassword.Text;
			}
		}

		public bool Spectator
		{
			get
			{
				return checkBox1.Checked;
			}
		}

		public BattleLoginPrompt(bool password)
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			if ( !password )
				textBoxPassword.Enabled = false;
			//
			// TODO: Add any constructor code after InitializeComponent call
			//
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		private bool ValidatePassword()
		{
			if ( !textBoxPassword.Enabled )
				return true;
			if ( textBoxPassword.Text.Length == 0 )
			{
				errorProviderPassword.SetError(textBoxPassword, "A password is needed");
				return false;
			}
			if ( textBoxPassword.Text.Length > CONSTANTS.MAXPASSWD )
			{
				errorProviderPassword.SetError(textBoxPassword, string.Format("Password cannot exceed {0} characters", CONSTANTS.MAXPASSWD));
				return false;
			}
			if ( Regex.Match(textBoxPassword.Text, "[0-9a-zA-Z]*", RegexOptions.None).Value != textBoxPassword.Text )
			{
				errorProviderPassword.SetError(textBoxPassword, "Password can contain only alphanumeric characters");
				return false;
			}
			return true;
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(BattleLoginPrompt));
			this.textBoxPassword = new System.Windows.Forms.TextBox();
			this.label1 = new System.Windows.Forms.Label();
			this.checkBox1 = new System.Windows.Forms.CheckBox();
			this.buttonOK = new System.Windows.Forms.Button();
			this.buttonCancel = new System.Windows.Forms.Button();
			this.errorProviderPassword = new System.Windows.Forms.ErrorProvider();
			this.SuspendLayout();
			// 
			// textBoxPassword
			// 
			this.textBoxPassword.Location = new System.Drawing.Point(136, 32);
			this.textBoxPassword.Name = "textBoxPassword";
			this.textBoxPassword.Size = new System.Drawing.Size(120, 20);
			this.textBoxPassword.TabIndex = 0;
			this.textBoxPassword.Text = "";
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(24, 32);
			this.label1.Name = "label1";
			this.label1.TabIndex = 1;
			this.label1.Text = "Password";
			// 
			// checkBox1
			// 
			this.checkBox1.Location = new System.Drawing.Point(72, 64);
			this.checkBox1.Name = "checkBox1";
			this.checkBox1.TabIndex = 2;
			this.checkBox1.Text = "Spectator";
			// 
			// buttonOK
			// 
			this.buttonOK.Location = new System.Drawing.Point(56, 112);
			this.buttonOK.Name = "buttonOK";
			this.buttonOK.TabIndex = 3;
			this.buttonOK.Text = "OK";
			this.buttonOK.Click += new System.EventHandler(this.buttonOK_Click);
			// 
			// buttonCancel
			// 
			this.buttonCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.buttonCancel.Location = new System.Drawing.Point(168, 112);
			this.buttonCancel.Name = "buttonCancel";
			this.buttonCancel.TabIndex = 4;
			this.buttonCancel.Text = "Cancel";
			this.buttonCancel.Click += new System.EventHandler(this.buttonCancel_Click);
			// 
			// errorProviderPassword
			// 
			this.errorProviderPassword.ContainerControl = this;
			// 
			// BattleLoginPrompt
			// 
			this.AcceptButton = this.buttonOK;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.CancelButton = this.buttonCancel;
			this.ClientSize = new System.Drawing.Size(292, 150);
			this.Controls.Add(this.buttonCancel);
			this.Controls.Add(this.buttonOK);
			this.Controls.Add(this.checkBox1);
			this.Controls.Add(this.label1);
			this.Controls.Add(this.textBoxPassword);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Name = "BattleLoginPrompt";
			this.Text = "BattleLoginPrompt";
			this.ResumeLayout(false);

		}
		#endregion

		private void buttonOK_Click(object sender, System.EventArgs e)
		{
			if ( !ValidatePassword() )
			{
				return;
			}
			DialogResult = DialogResult.OK;
			Close();
		}

		private void buttonCancel_Click(object sender, System.EventArgs e)
		{
			DialogResult = DialogResult.Cancel;
			Close();
		}
	}
}
