using System;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Windows.Forms;
using System.Collections;
using System.Text;

namespace SpringClient
{
	/// <summary>
	/// Summary description for SocketHandler.
	/// </summary>
	public class SocketHandler
	{
		private Socket		m_Socket;
		private int			m_ID;
		public delegate void delReadID(byte[] read, int id);
		public delegate void delRead(byte[] read);
		public delegate void delSend();

		private class StateReadObject
		{
			public Socket s;
			public byte[] buffer;
			public int offset;
			public int toRead;
			public int id;
			public delRead ReadCallback;
			public delReadID ReadIDCallback;
			public Control calledFrom;
		}

		private class StateSendObject
		{
			public Socket s;
			public byte[] buffer;
			public int offset;
			public int toSend;
			public delSend SendCallback;
			public Control calledFrom;
		}

		public bool Connected
		{
			get
			{
				return m_Socket.Connected;
			}
		}
		public SocketHandler(Socket s)
		{
			//
			// TODO: Add constructor logic here
			//
			m_Socket = s;
		}

		public void SetID(int id)
		{
			m_ID = id;
		}

		public void Read(int amount, delReadID readCallback, Control calledFrom)
		{
			try
			{
				StateReadObject so = new StateReadObject();
				so.buffer = new byte[amount];
				so.s = m_Socket;
				so.toRead = amount;
				so.ReadIDCallback = readCallback;
				so.offset = 0;
				so.calledFrom = calledFrom;
				so.id = m_ID;

				IAsyncResult iar = m_Socket.BeginReceive(so.buffer, so.offset, amount, SocketFlags.None, new AsyncCallback(ReadIDCallback), so);
			}
			catch ( Exception ex )
			{
				throw ex;
			}
		}

		public void Read(int amount, delRead readCallback, Control calledFrom)
		{
			try
			{
				//	System.Diagnostics.Debug.WriteLine("BLAH");
				StateReadObject so = new StateReadObject();
				so.buffer = new byte[amount];
				so.s = m_Socket;
				so.toRead = amount;
				so.ReadCallback = readCallback;
				so.offset = 0;
				so.calledFrom = calledFrom;
				IAsyncResult iar = m_Socket.BeginReceive(so.buffer, so.offset, amount, SocketFlags.None, new AsyncCallback(ReadCallback), so);
			}
			catch ( ObjectDisposedException )
			{
			}
			catch ( SocketException )
			{
			}
			catch ( Exception e )
			{
				throw e;
			}
		}

		private void ReadCallback( IAsyncResult iar )
		{
			try
			{
		//		System.Diagnostics.Process process = System.Diagnostics.Process.GetCurrentProcess();
		//		System.Diagnostics.Debug.WriteLine(string.Format("Reading {0}", process.Threads.Count));
				
				StateReadObject so = (StateReadObject)iar.AsyncState;
				int read = so.s.EndReceive(iar);
				so.offset += read;
				so.toRead -= read;
				if ( so.toRead > 0 )
				{
					System.Diagnostics.Debug.WriteLine("\tMORE to Read" + so.toRead.ToString());
					// continue reading until all the data is received
					so.s.BeginReceive(so.buffer, so.offset, so.toRead, SocketFlags.None, new AsyncCallback(ReadCallback), so);
					return;
				}
				else
					// callback when the buffer is fully read
					so.calledFrom.BeginInvoke(so.ReadCallback, new object[] {so.buffer});
			}
			catch ( SocketException )
			{
			}
			catch ( ObjectDisposedException )
			{
				// this will happen when you're closing the connection... so we won't worry about it
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
			}
		}

		private void ReadIDCallback( IAsyncResult iar )
		{
			try
			{
				StateReadObject so = (StateReadObject)iar.AsyncState;
				int read = so.s.EndReceive(iar);
				so.offset += read;
				so.toRead -= read;
				if ( so.toRead > 0 )
				{
					// continue reading until all the data is received
					so.s.BeginReceive(so.buffer, so.offset, so.toRead, SocketFlags.None, new AsyncCallback(ReadIDCallback), so);
					return;
				}
				else
					// callback when the buffer is fully read
					so.calledFrom.BeginInvoke(so.ReadIDCallback, new object[] {so.buffer, so.id});
			}
			catch ( SocketException )
			{
				// same here
			}
			catch ( ObjectDisposedException )
			{
				// this will happen when you're closing the connection... so we won't worry about it
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
			}
		}

		public void Send( byte[] data )
		{
			Send(data, 0, data.Length, null, null);
		}
		public void Send( byte[] data, delSend SendCallback, Control calledFrom )
		{
			Send(data, 0, data.Length, SendCallback, calledFrom );
		}

		public void Send( int data )
		{
			Send(data, null, null);
		}

		public void Send( int data, delSend SendCallback, Control calledFrom )
		{
			byte[] buffer = BitConverter.GetBytes(data);
			Send(buffer, SendCallback, calledFrom);
		}

		public void Send( uint data )
		{
			Send(data, null, null);
		}

		public void Send( uint data, delSend SendCallback, Control calledFrom )
		{
			byte[] buffer = BitConverter.GetBytes(data);
			Send(buffer, SendCallback, calledFrom);
		}

		public void Send( double data )
		{
			Send(data, null, null);
		}

		public void Send( double data, delSend SendCallback, Control calledFrom )
		{
			byte[] buffer = BitConverter.GetBytes(data);
			Send(buffer, SendCallback, calledFrom);
		}

		public void Send( float data )
		{
			Send(data, null, null);
		}

		public void Send( float data, delSend SendCallback, Control calledFrom )
		{
			byte[] buffer = BitConverter.GetBytes(data);
			Send(buffer, SendCallback, calledFrom);
		}

		public void Send( short data )
		{
			Send(data, null, null);
		}

		public void Send( short data, delSend SendCallback, Control calledFrom )
		{
			byte[] buffer = BitConverter.GetBytes(data);
			Send(buffer, SendCallback, calledFrom);
		}

		public void Send( ushort data )
		{
			Send(data, null, null);
		}

		public void Send( ushort data, delSend SendCallback, Control calledFrom )
		{
			byte[] buffer = BitConverter.GetBytes(data);
			Send(buffer, SendCallback, calledFrom);
		}

		public void Send( byte data )
		{
			Send(data, null, null);
		}

		public void Send( byte data, delSend SendCallback, Control calledFrom )
		{
			byte[] buffer = new byte[1];
			buffer[0] = data;
			Send(buffer, SendCallback, calledFrom);
		}

		public void Send( long data )
		{
			Send(data, null, null);
		}

		public void Send( long data, delSend SendCallback, Control calledFrom )
		{
			byte[] buffer = BitConverter.GetBytes(data);
			Send(buffer, SendCallback, calledFrom);
		}

		public void Send( ulong data )
		{
			Send(data, null, null);
		}

		public void Send( ulong data, delSend SendCallback, Control calledFrom )
		{
			byte[] buffer = BitConverter.GetBytes(data);
			Send(buffer, SendCallback, calledFrom);
		}

		public void Send( string data )
		{
			Send(data, null, null);
		}

		public void Send( string data, delSend SendCallback, Control calledFrom )
		{
			byte[] buffer = Encoding.Default.GetBytes(data);
			Send(buffer, SendCallback, calledFrom);
		}

		public void Send( byte[] data, int offset, int amount )
		{
			Send( data, offset, amount, null, null);
		}

		public void Send( byte[] data, int offset, int amount, delSend sendCallback, Control calledFrom )
		{
			try
			{
				StateSendObject sro = new StateSendObject();
				sro.buffer = data;
				sro.calledFrom = calledFrom;
				sro.offset = offset;
				sro.s = m_Socket;
				sro.SendCallback = sendCallback;
				sro.toSend = amount;

				m_Socket.BeginSend(sro.buffer, sro.offset, sro.toSend, SocketFlags.None, new AsyncCallback(SendCallback), sro);
			}
			catch ( SocketException )
			{
			}
			catch ( Exception ex )
			{
				throw ex;
			}
		}

		private void SendCallback( IAsyncResult iar )
		{
			try
			{
				StateSendObject so = (StateSendObject)iar.AsyncState;
				int sent = so.s.EndReceive(iar);
				so.offset += sent;
				so.toSend -= sent;
				if ( so.toSend > 0 )
				{
					// continue sending until all data has been sent
					so.s.BeginSend(so.buffer, so.offset, so.toSend, SocketFlags.None, new AsyncCallback(SendCallback), so);
					return;
				}
				else
				{
					// if there was no callback on complete, then just return
					if ( so.SendCallback == null )
						return;
					// else, call the callback
					so.calledFrom.BeginInvoke(so.SendCallback, null);
				}
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
			}
		}
	}
}