// SunParser.cpp: implementation of the CSunParser class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
//#include "SunParser.h"
#include <stdio.h>
#include <stdarg.h>
#include <algorithm>
#include <cctype>
#include ".\sunparser.h"

//#include "mmgr.h"

#pragma warning(disable:4786)
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

void* hWnd = 0;
#ifdef MB_OK
#undef MB_OK
#endif
#define MB_OK 0

void MessageBox(void* hWnd, const char* lpTest, const char* lpCaption, unsigned int uType)
{
//	CLog::Error("Message box - \"%s\" with caption \"%s\"", __FUNCTION__, lpTest, lpCaption); 
}

CSunParser::CSunParser()
{

}

CSunParser::~CSunParser()
{
	DeleteSection(&sections);
}

void CSunParser::DeleteSection(std::map<std::string,SSection*> *section)
{
	std::map<std::string, SSection*>::iterator ui;
	for(ui=section->begin();ui!=section->end();ui++)
	{
		DeleteSection(&ui->second->sections);
		delete ui->second;
	}
}

void CSunParser::LoadFile(std::string filename)
{
	//DeleteSection(&sections);

	this->filename = filename;

	std::ifstream inFile(filename.c_str(), std::ios_base::binary);

//	CFileHandler file(filename);
	if(!inFile.is_open()){
		MessageBox(hWnd, ("file " + filename + " not found").c_str(), "file error", MB_OK);
		return;
	}
	inFile.seekg(0, std::ios_base::end);
	int size = inFile.tellg();
	inFile.seekg(0);
	char *filebuf = new char[size+1];

	inFile.read(filebuf, size);
	inFile.close();
	
//	file.Read(filebuf, file.FileSize());
	
	filebuf[size] = '\n'; //append newline at end to avoid parsing error at eof
	
	try
	{
		Parse(filebuf, size);
	}
	catch(...)
	{
		MessageBox(hWnd, ("error parsing file " + filename).c_str(), "Sun parsing error", MB_OK);
	}

	delete[] filebuf;
}

void CSunParser::Parse(char *buf, int size)
{
	std::string thissection;
	SSection *section = NULL;

	//std::vector<std::map<std::string,SSection*>*> sectionlist;
	//sectionlist.push_back(&sections);

	int se = 0; //for section start/end errorchecking

	char *endptr = buf+size;

	while(buf!=endptr)
	{
		if(buf[0]=='/' && buf[1]=='/') //comment
		{
			while(*buf!='\n' && *buf!='\r')
			{
				buf++;
			}
		}
		else if(buf[0]=='/' && buf[1]=='*') //comment
		{
			while(buf[0]!='*' || buf[1]!='/')
			{
				buf++;
			}
		}
		else if(*buf == '[') //sectionname
		{
			thissection = "";
			while(*(++buf)!=']')
			{
				thissection += *buf;
			}
		}
		else if(*buf == '{') //section
		{
			buf++;
			section = new SSection;
			std::transform(thissection.begin(), thissection.end(), thissection.begin(), (int (*)(int))std::tolower);
			std::map<std::string, SSection*>::iterator ui=sections.find(thissection);
			if(ui!=sections.end()){
//				MessageBox(0,"Overwrote earlier section in sunparser",thissection.c_str(),0);
				DeleteSection(&ui->second->sections);
				delete ui->second;
			}
			sections[thissection] = section;
			buf = ParseSection(buf, (int)(endptr-buf), section);

		}
		buf++;
	}
}

char *CSunParser::ParseSection(char *buf, int size, SSection *section)
{
	std::string thissection;

	int se = 0; //for section start/end errorchecking

	char *endptr = buf+size;

	while(buf!=endptr)
	{
		if(buf[0]=='/' && buf[1]=='/') //comment
		{
			while(*buf!='\n' && *buf!='\r')
			{
				buf++;
			}
		}
		else if(buf[0]=='/' && buf[1]=='*') //comment
		{
			while(buf[0]!='*' || buf[1]!='/')
			{
				buf++;
			}
		}
		else if(*buf == '[') //sectionname
		{
			thissection = "";
			while(*(++buf)!=']')
			{
				thissection += *buf;
			}
		}
		else if(*buf == '{') //section
		{
			buf++;
			SSection *newsection = new SSection;
			std::transform(thissection.begin(), thissection.end(), thissection.begin(), (int (*)(int))std::tolower);
			std::map<std::string, SSection*>::iterator ui=section->sections.find(thissection);
			if(ui!=section->sections.end()){
				MessageBox((void*)0,"Overwrote earlier section in sunparser",thissection.c_str(),0);
				DeleteSection(&ui->second->sections);
				delete ui->second;
			}
			section->sections[thissection] = newsection;
			buf = ParseSection(buf, (int)(endptr-buf), newsection);
		}
		else if(*buf == '}') //endsection
		{
			//buf++;
			return buf;
		}
		else if(*buf >= '0' && *buf <= 'z')
		{
			std::string name;
			std::string value;

			while(*buf != '=')
			{
				name += *buf;
				buf++;
			}
			buf++;
			while(*buf != ';')
			{
				value += *buf;
				buf++;
			}

			std::transform(name.begin(), name.end(), name.begin(), (int (*)(int))std::tolower);
			section->values[name] = value;
		}

		buf++;
	}

	return buf;
}

//find value, display messagebox if no such value found
std::string CSunParser::SGetValueMSG(std::string location)
{
	std::transform(location.begin(), location.end(), location.begin(), (int (*)(int))std::tolower);
	std::string value;

	bool found = SGetValue(value, location);

	if(!found)
		MessageBox(hWnd, value.c_str(), "Sun parsing error", MB_OK);

	return value;
}

//find value, return default value if no such value found
std::string CSunParser::SGetValueDef(std::string defaultvalue, std::string location)
{
	std::transform(location.begin(), location.end(), location.begin(), (int (*)(int))std::tolower);
	std::string value;

	bool found = SGetValue(value, location);

	if(!found)
		value = defaultvalue;

	return value;
}

//finds a value in the file, if not found returns false, and errormessages is returned in value
bool CSunParser::GetValue(std::string &value, ...)
{

	std::string searchpath; //for errormessages

	va_list loc;
	va_start(loc, value);

	int numargs=0;
	while(va_arg(loc, char*)) //determine number of arguments
		numargs++;

	va_start(loc, value);
	
	SSection *sectionptr;
	for(int i=0; i<numargs-1; i++)
	{
		char *arg = va_arg(loc, char*);

		searchpath += '\\';
		searchpath += arg;

		sectionptr = sections[arg];
		if(sectionptr==NULL)
		{
			value = "Section " + searchpath + " missing in file " + filename;

			return false;
		}
	}

	char *arg = va_arg(loc, char*);
	std::string svalue = sectionptr->values[arg];

	searchpath += '\\';
	searchpath += arg;

	if(svalue == "")
	{
		value = "Value " + searchpath + " missing in file " + filename;

		return false;
	}

	value = svalue;
	return true;

}

void CSunParser::Test()
{
	SSection *unitinfo = sections["UNITINFO"];
	SSection *weapons = unitinfo->sections["WEAPONS"];

	std::string mo = weapons->values["weapon1"];
}
//finds a value in the file , if not found returns false, and errormessages is returned in value
//location of value is sent as a string "section\section\value"
bool CSunParser::SGetValue(std::string &value, std::string location)
{
	std::transform(location.begin(), location.end(), location.begin(), (int (*)(int))std::tolower);
	std::string searchpath; //for errormessages

	//split the location string
	std::vector<std::string> loclist = GetLocationVector(location);
		
	if(sections.find(loclist[0]) == sections.end())
	{
		value = "Section " + loclist[0] + " missing in file " + filename;
		return false;
	}
	SSection *sectionptr = sections[loclist[0]];
	searchpath = loclist[0];
	for(unsigned int i=1; i<loclist.size()-1; i++)
	{
		//const char *arg = loclist[i].c_str();

		searchpath += '\\';
		searchpath += loclist[i];

		if(sectionptr->sections.find(loclist[i]) == sectionptr->sections.end())
		{
			value = "Section " + searchpath + " missing in file " + filename;

			return false;
		}
		sectionptr = sectionptr->sections[loclist[i]];
	}

	searchpath += '\\';
	searchpath += loclist[loclist.size()-1];

	if(sectionptr->values.find(loclist[loclist.size()-1]) == sectionptr->values.end())
	{
		value = "Value " + searchpath + " missing in file " + filename;

		return false;
	}

	std::string svalue = sectionptr->values[loclist[loclist.size()-1]];
	value = svalue;
	return true;

}

//return a map with all values in section
const std::map<std::string, std::string> CSunParser::GetAllValues(std::string location)
{
	std::transform(location.begin(), location.end(), location.begin(), (int (*)(int))std::tolower);
	std::map<std::string, std::string> emptymap;
	std::string searchpath; //for errormessages

	std::vector<std::string> loclist = GetLocationVector(location);	


	if(sections.find(loclist[0]) == sections.end())
	{
		MessageBox(hWnd, ("Section " + loclist[0] + " missing in file " + filename).c_str(), "Sun parsing error", MB_OK);
		return emptymap;
	}
	SSection *sectionptr = sections[loclist[0]];
	searchpath = loclist[0];
	for(unsigned int i=1; i<loclist.size(); i++)
	{
		searchpath += '\\';
		searchpath += loclist[i];

		if(sectionptr->sections.find(loclist[i]) == sectionptr->sections.end())
		{
			MessageBox(hWnd, ("Section " + searchpath + " missing in file " + filename).c_str(), "Sun parsing error", MB_OK);
			return emptymap;
		}

		sectionptr = sectionptr->sections[loclist[i]];
	}

	return sectionptr->values;
}

//return vector with all section names in it
std::vector<std::string> CSunParser::GetSectionList(std::string location)
{
	std::transform(location.begin(), location.end(), location.begin(), (int (*)(int))std::tolower);
	std::vector<std::string> loclist = GetLocationVector(location);

	std::vector<std::string> returnvec;

	std::map<std::string,SSection*> *sectionsptr = &sections;

	if(loclist[0].compare("")!=0)
	{
 		std::string searchpath;// = loclist[0];
		for(unsigned int i=0; i<loclist.size(); i++)
		{
			searchpath += loclist[i];

			if(sectionsptr->find(loclist[i]) == sectionsptr->end())
			{
				MessageBox(hWnd, ("Section " + searchpath + " missing in file " + filename).c_str(), "Sun parsing error", MB_OK);
				return returnvec;
			}

			sectionsptr = &sectionsptr->find(loclist[i])->second->sections;
			
			searchpath += '\\';
		}       
	}

	std::map<std::string,SSection*>::iterator it;
	for(it=sectionsptr->begin(); it!=sectionsptr->end(); it++)
	{
		returnvec.push_back(it->first);
		std::transform(returnvec.back().begin(), returnvec.back().end(), returnvec.back().begin(), (int (*)(int))std::tolower);
	}

	return returnvec;
}

bool CSunParser::SectionExist(std::string location)
{
	std::transform(location.begin(), location.end(), location.begin(), (int (*)(int))std::tolower);
	std::vector<std::string> loclist = GetLocationVector(location);		
	
	if(sections.find(loclist[0]) == sections.end())
	{
		return false;
	}
	SSection *sectionptr = sections[loclist[0]];
	for(unsigned int i=1; i<loclist.size(); i++)
	{
		if(sectionptr->sections.find(loclist[i]) == sectionptr->sections.end())
		{
			return false;
		}

		sectionptr = sectionptr->sections[loclist[i]];
	}

	return true;
}

std::vector<std::string> CSunParser::GetLocationVector(std::string location)
{
	std::transform(location.begin(), location.end(), location.begin(), (int (*)(int))std::tolower);
	std::vector<std::string> loclist;
	size_t start = 0;
	size_t next = 0;
	static const std::basic_string <char>::size_type npos = -1;

	while((next = location.find_first_of("\\", start)) != npos)
	{
		loclist.push_back(location.substr(start, next-start));
		start = next+1;
	}
	loclist.push_back(location.substr(start, -1));

    return loclist;
}

/*
template<typename T>
void CSunParser::GetMsg(T& value, const std::string& key)
{
	std::string str;
	str = SGetValueMSG(key);

	std::stringstream stream;
	stream << str;
	stream >> value;
}

template<typename T>
void CSunParser::GetDef(T& value, const std::string& key, const std::string& defvalue)
{
	std::string str;
	str = SGetValueDef(key, defvalue);

	std::stringstream stream;
	stream << str;
	stream >> value;
	}*/

float3 CSunParser::GetFloat3(float3 def, std::string location)
{
	std::string s=SGetValueDef("",location);
	if(s.empty())
		return def;

	float3 ret;
	ParseArray(s,ret.xyz,3);
	return ret;
}
