#include "stdafx.h"
#include "MoveMath.h"
#include "ReadMap.h"
#include "feature.h"
#include "unit.h"
#include "commandai.h"
//#include "mmgr.h"


/*
Converts a point-request into a square-positional request.
*/
float CMoveMath::SpeedMod(const MoveData& moveData, float3 pos) {
	int x = pos.x / SQUARE_SIZE;
	int z = pos.z / SQUARE_SIZE;
	return SpeedMod(moveData, x, z);
}


/*
Extract height and slope data from given square and make
a sub-functional call for calculations upon whose data.
*/
float CMoveMath::SpeedMod(const MoveData& moveData, int xSquare, int zSquare) {
	//Error-check
	if(xSquare < 0 || zSquare < 0
	|| xSquare >= gs->mapx || zSquare >= gs->mapy)
		return 0.0f;

	//Extract data.
	int square = xSquare/2 + zSquare/2 * gs->hmapx;
	float height = readmap->halfHeightmap[square];
	float slope = readmap->slopemap[square];

	//Call calculations.
	return SpeedMod(moveData, height, slope);
}

float CMoveMath::SpeedMod(const MoveData& moveData, float3 pos,const float3& moveDir) {
	int x = (int)pos.x / SQUARE_SIZE;
	int z = (int)pos.z / SQUARE_SIZE;
	return SpeedMod(moveData, x, z,moveDir);
}


/*
Extract height and slope data from given square and make
a sub-functional call for calculations upon whose data.
*/
float CMoveMath::SpeedMod(const MoveData& moveData, int xSquare, int zSquare,const float3& moveDir) {
	//Error-check
	if(xSquare < 0 || zSquare < 0
	|| xSquare >= gs->mapx || zSquare >= gs->mapy)
		return 0.0f;

	//Extract data.
	int square = xSquare/2 + zSquare/2 * gs->hmapx;
	float height = readmap->halfHeightmap[square];
	float slope = readmap->slopemap[square];

	float3 flatNorm=readmap->facenormals[(xSquare + zSquare * gs->mapx)*2];
	flatNorm.y=0;
	flatNorm.Normalize();
	float moveSlope=-moveDir.dot(flatNorm);

	//Call calculations.
	return SpeedMod(moveData, height, slope,moveSlope);
}

/*
Converts a point-request into a square-positional request.
*/
int CMoveMath::IsBlocked(const MoveData& moveData, float3 pos) {
	int x = pos.x / SQUARE_SIZE;
	int z = pos.z / SQUARE_SIZE;
	return IsBlocked(moveData, x, z);
}

/*
Check if a given square-position is accessable by the footprint.
this one check terrain
*/
int CMoveMath::IsBlocked(const MoveData& moveData, int xSquare, int zSquare) {
	if(CMoveMath::SpeedMod(moveData, xSquare, zSquare) == 0.0f)
		return true;

	int ret=0;

	ret|=SquareIsBlocked(moveData, xSquare, zSquare);
	ret|=SquareIsBlocked(moveData, xSquare - moveData.size/2, zSquare - moveData.size/2);
	ret|=SquareIsBlocked(moveData, xSquare + moveData.size/2 - 1, zSquare - moveData.size/2);
	ret|=SquareIsBlocked(moveData, xSquare - moveData.size/2, zSquare + moveData.size/2 - 1);
	ret|=SquareIsBlocked(moveData, xSquare + moveData.size/2 - 1, zSquare + moveData.size/2 - 1);

	return ret;
}

/*
Converts a point-request into a square-positional request.
*/
int CMoveMath::IsBlocked2(const MoveData& moveData, float3 pos) {
	int x = pos.x / SQUARE_SIZE;
	int z = pos.z / SQUARE_SIZE;
	return IsBlocked2(moveData, x, z);
}

/*
Check if a given square-position is accessable by the footprint.
this one doesnt check terrain
but do check so it doesnt run over something small if its big
*/
int CMoveMath::IsBlocked2(const MoveData& moveData, int xSquare, int zSquare) {
	int ret=0;

	ret|=SquareIsBlocked(moveData, xSquare, zSquare);
	ret|=SquareIsBlocked(moveData, xSquare - moveData.size/2, zSquare - moveData.size/2);
	ret|=SquareIsBlocked(moveData, xSquare + moveData.size/2-1, zSquare - moveData.size/2);
	ret|=SquareIsBlocked(moveData, xSquare - moveData.size/2, zSquare + moveData.size/2-1);
	ret|=SquareIsBlocked(moveData, xSquare + moveData.size/2-1, zSquare + moveData.size/2-1);
	if(moveData.size>4){
		ret|=SquareIsBlocked(moveData, xSquare -1, zSquare - 1);
		if(moveData.size>6){
			ret|=SquareIsBlocked(moveData, xSquare - moveData.size/2+1, zSquare - moveData.size/2+1);
			ret|=SquareIsBlocked(moveData, xSquare + moveData.size/2-2, zSquare - moveData.size/2+1);
			ret|=SquareIsBlocked(moveData, xSquare - moveData.size/2+1, zSquare + moveData.size/2-2);
			ret|=SquareIsBlocked(moveData, xSquare + moveData.size/2-3, zSquare + moveData.size/2-3);
			if(moveData.size>10){
				ret|=SquareIsBlocked(moveData, xSquare - moveData.size/2+2, zSquare - moveData.size/2+2);
				ret|=SquareIsBlocked(moveData, xSquare + moveData.size/2-3, zSquare - moveData.size/2+2);
				ret|=SquareIsBlocked(moveData, xSquare - moveData.size/2+2, zSquare + moveData.size/2-3);
				ret|=SquareIsBlocked(moveData, xSquare + moveData.size/2-3, zSquare + moveData.size/2-3);
			}
		}
	}
	return ret;
}

/*
Simply check if the object is blocking or not.
*/
bool CMoveMath::IsBlocking(const MoveData& moveData, const CSolidObject* object) {
	return object->blocking && (!dynamic_cast<const CFeature*>(object) || object->mass > moveData.crushStrength);
}


/*
Converts a point-request into a square-positional request.
*/
float CMoveMath::yLevel(const float3 pos) {
	int x = pos.x / SQUARE_SIZE;
	int z = pos.z / SQUARE_SIZE;
	return yLevel(x, z);
}

/*
Check if a single square is accessable.
*/
int CMoveMath::SquareIsBlocked(const MoveData& moveData, int xSquare, int zSquare) {
	//Error-check
	if(xSquare < 0 || zSquare < 0
	|| xSquare >= gs->mapx || zSquare >= gs->mapy)
		return true;

	CSolidObject* obstacle = readmap->GroundBlocked(xSquare + zSquare * gs->mapx);
	if(obstacle) {
		if(obstacle->mobility) {
			if(obstacle->isMoving){
				return BLOCK_MOVING;
			} else {
				if(((CUnit*)obstacle)->commandAI->commandQue.empty()){
					return BLOCK_MOBILE;
				} else {
					return BLOCK_MOBILE_BUSY;
				}
			}
		} else {
			if(IsBlocking(moveData, obstacle))
				return BLOCK_STRUCTURE;
		}
	}
  return 0;
}

